package weka.core;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;

/* Inherit from some arbitrary Collection-implementing class  */

public class Attributes extends Object {
  
  private ArrayList attributes;

  public Attributes(){
    attributes = new ArrayList();
  }

  public Attributes(Attributes attrs) {
    attributes = new ArrayList();
    attributes.addAll(attrs.getAttributes());
  }

  public Attributes(Instances i) {
    attributes = new ArrayList(i.numAttributes());
    Enumeration e = i.enumerateAttributes();
    while ( e.hasMoreElements() ) {
      add((Attribute)e.nextElement());
    } // end of while ()
  }

  public Attributes(Attributes a,int[]combination){
    attributes = new ArrayList(combination.length);
    for (int i=0;i<combination.length;i++){
      attributes.add(a.get(combination[i]));
    }
  }

  public static Attributes nominalAttributes(Instances insts){
    Attributes a = new Attributes();
    for (int i=0;i<insts.numAttributes();i++) {
      if ( insts.attribute(i).isNominal() && i != insts.classIndex() ) {
	a.add(insts.attribute(i));
      } // end of if ()
    } // end of for ()
    return a;
  }

  public static Attributes equalAttributes(Instance i1, Instance i2) {
    Attributes a = new Attributes();
    if ( i1.dataset().equalHeaders(i2.dataset()) && 
	 i1.dataset().checkInstance(i2)) {
      for (int i=0;i<i1.numAttributes();i++) {
	if (!(i1.isMissing(i) || i2.isMissing(i)) 
	    && (i1.value(i) == i2.value(i))) {
	  a.add(i1.attribute(i));
	} // end of if ()
      } // end of for ()
    } // end of if ()
    return a;
  }

  public boolean subsetOf(Attributes a) {
    return a.getAttributes().containsAll(attributes);
  }

  public Collection getAttributes() {
    return attributes;
  }

  public void add(Attribute a)  {
    attributes.add(a); 
  }

  public void add(int i,Attribute a)  {
    attributes.add(i,a); 
  }

  public void remove(Attribute a){
    attributes.remove(a);
  }

  public void remove(int i){
    attributes.remove(i);
  }

  public Attribute get(int i) {
    return (Attribute)attributes.get(i);
  }


  public void clear(){
    attributes.clear();
  }
  
  public Attributes removeAll(Attributes a) {
    Iterator i = a.iterator();
    Attributes res = new Attributes();
    while ( i.hasNext() ) {
      Attribute attr = (Attribute)i.next();
      attributes.remove(attr);	    
      res.add(attr);
    } // end of while ()
    return res;
  }

  
  public Iterator iterator() {
    return attributes.iterator();
  }

  public int size() {
    return attributes.size();
  }

  public boolean isEmpty() {
    return attributes.isEmpty();
  }

  public boolean equals(Object o) {
    if ( !(o instanceof Attributes) ) {
      return false;
    } // end of if ()
    return this.subsetOf((Attributes)o) && ((Attributes)o).subsetOf(this);
  }

  public int hashCode(){
    int sum=0;
    for (Iterator i = iterator();i.hasNext(); ) {
      sum += ((Attribute)i.next()).toString().hashCode();
    } // end of for ()
    return sum;
  }
  
  public String toString(){
    Iterator i = iterator();
    StringBuffer s = new StringBuffer();
    while ( i.hasNext() ) {
      s.append(((Attribute)i.next()).name());
      if ( i.hasNext() ) {
	s.append(", ");	
      } // end of if ()
    } // end of while ()
    return s.toString();
  }

  public FastVector toFastVector() {
    FastVector f = new FastVector(this.size());
    for (Iterator i=this.iterator();i.hasNext(); ) {
      f.addElement((Attribute)i.next());
    } // end of for ()
    return f;
  }

}


