/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *    Permutations.java
 *    Copyright (C) 2002 Ola Leifler
 *
 */

package weka.core;

import java.io.*;
import java.util.*;


public class Permutations {

  class Permutation extends Object implements Comparable {
    
    private int[] perm = null;

    public Permutation(int[] p){
      perm=p;
    }

    public int hashCode(){
      return this.toString().hashCode();
    }

    public boolean equals(Object o){
      return Arrays.equals(perm,((Permutation)o).getPermutation());
    }

    public int compareTo(Object o){
      int[] p = ((Permutation)o).getPermutation();
      int comp=0;
      for (int i=0;i<p.length && comp==0;i++) {
	comp = perm[i]-p[i];
      } // end of for ()
      return comp;
    }

    public int[] getPermutation(){
      return perm;
    }

    public String toString(){
      StringBuffer s = new StringBuffer();
      for (int i=0;i<perm.length;i++) {
	s.append(perm[i]+(i<perm.length-1?",":"\n"));
      } // end of for ()
      return s.toString();
    }

  }

  private TreeSet oldPerms = null;
  private Permutation currentPerm = null;
  private boolean moreElements = true;

  public Permutations(int n) {
    int[] perm = new int[n];
    for (int i=0;i<perm.length;i++) {
      perm[i] = i;
    } // end of for ()
    currentPerm = new Permutation(perm);
    oldPerms = new TreeSet();
    moreElements = n>1;
    oldPerms.add(currentPerm);
  }
  

  public boolean hasNext() {
    return moreElements;
  }

  public int[] next() {
    int[] res = currentPerm.getPermutation();
    moreElements = oldPerms.size() < Utils.fac(currentPerm.getPermutation().
					       length);
    if ( moreElements ) {
      int [] perm = currentPerm.getPermutation();
      int n = perm.length;
      int [] next = new int[n];
      System.arraycopy(perm,0,next,0,perm.length);
      Permutation nextPerm;
      do {
	for (int j=n-1;j>0;j--) {
	  int r = (int) (n * Math.random());
	  int tmp = next[r];
	  next[r] = next[j];
	  next[j] = tmp;
	} // end of for ()
	nextPerm = new Permutation(next);
      } while ( oldPerms.contains(nextPerm));
      currentPerm = nextPerm;  
      oldPerms.add(currentPerm);
    } // end of if()
    return res;
  }

  public static void main (String[] args) {
    Permutations p = new Permutations(4);
    System.out.println("Permutations of 4 objects: ");
    while ( p.hasNext() ) {p.next();} // end of while ()
    System.out.println(p.toString());
  } // end of main ()


  public String toString(){
    StringBuffer s = new StringBuffer();
    for (Iterator iter=oldPerms.iterator();iter.hasNext(); ) {
      s.append(((Permutation)iter.next()).toString());
    } // end of for ()
    return s.toString();
  }  
}
