 /*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * AttributeValuePairs
 * Copyleft (C) 2002 Ola Leifler
 *
 */


package weka.roughset;

import weka.core.*;
import java.io.*;
import java.util.*;

public class AttributeValuePair extends Object implements Comparable {
  
  private Attribute attribute;
  private double value;

  public AttributeValuePair() {
    attribute = new Attribute("Dummy");
    value = Double.NaN;
  }

  public AttributeValuePair(Attribute a, double v){
    value = v;
    attribute = a;
  }
  
  public AttributeValuePair(AttributeValuePair p){
    value = p.getValue();
    attribute = p.getAttribute();
  }

  public AttributeValuePair(Instance i){
    value = i.classValue();
    attribute = i.classAttribute();
  }

  public int compareTo(Object o) {
    AttributeValuePair aValPair = (AttributeValuePair)o;
    int comp1  = attribute.name().
	compareTo(aValPair.getAttribute().name());
    int comp2 = (new Double(value)).
	compareTo(new Double(aValPair.getValue()));
    return (comp1 != 0 ? comp1 : comp2);
  }

  public boolean equals(Object o) {
    return compareTo(o) == 0;
  }

  public double getValue(){
    return value;
  }

  public void setValue(double v){
    value = v;
  }

  public Attribute getAttribute(){
    return attribute;
  }
  
  public void setAttribute(Attribute a){
    attribute = a;
  }

  public boolean matches(RoughInstance i) {
    return ( !Double.isNaN(value) ) && (i.value(attribute) == value);
  }

  public int hashCode() {
    return this.toString().hashCode();
  }

  public String toString(){
    StringBuffer s = new StringBuffer();
    s.append("(" + attribute.name() + "," + 
	     (attribute.isNumeric() ? 
	      (new Double(value)).toString() : attribute.value((int)value))
	     + ")");
    return s.toString();
  }

  public String ruleString(){
    StringBuffer s = new StringBuffer();
    s.append(attribute.name() + " = " 
	     + (attribute.isNumeric() ? 
		(new Double(value)).toString() : 
		attribute.value((int)value)));
    return s.toString();
  }

}
