/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *    LocalCovering.java
 *    Copyright (C) 2002 Ola Leifler
 *
 */

package weka.roughset;

import java.io.*;
import java.util.*;
import weka.core.*;

public class LocalCovering {

  private ArrayList pairSet;
  private Instances instances;

  public LocalCovering(Instances i){
    pairSet = new ArrayList();
    instances = i;
  }

  public LocalCovering(LocalCovering l) {
    pairSet = new ArrayList(l.getPairSet());
    instances = l.getInstances();
  }

  /**
   * Get the value of instances.
   * @return value of instances.
   */
  public Instances getInstances() {
    return instances;
  }
  
  /**
   * Set the value of instances.
   * @param v  Value to assign to instances.
   */
  public void setInstances(Instances  v) {
    this.instances = v;
  }
  

  /**
   * Get the value of pairSet.
   * @return value of pairSet.
   */
  public ArrayList getPairSet() {
    return pairSet;
  }
  
  /**
   * Set the value of pairSet.
   * @param v  Value to assign to pairSet.
   */

  public void setPairSet(ArrayList v) {
    this.pairSet = v;
  }
  

  public void addPairs(AttributeValuePairs p) {
    pairSet.add(p);
  }

  public void removePairs(AttributeValuePairs p) {
    pairSet.remove(pairSet.indexOf(p));
  }
  
  public boolean matches(RoughInstance r) {
    Iterator i = pairSet.iterator();
    boolean comp = false;
    while ( i.hasNext() ) {
      AttributeValuePairs pairs = (AttributeValuePairs)i.next();
      if ( pairs.matches(r) ) {
	comp = true;
	break;
      } // end of if ()
    } // end of while ()
    return comp;
  }


  public void trimToFit(RoughInstances r) {
    boolean b = r.isCompareAll();
    r.setCompareAll(true);
    Iterator i = pairSet.iterator();
    while ( i.hasNext() ) {
      AttributeValuePairs pairs = (AttributeValuePairs)i.next();
      LocalCovering l = new LocalCovering(this);
      l.removePairs(pairs);
      RoughInstances rough = new RoughInstances(instances);
      rough.retainMatching(l);
      if ( rough.equals(r) ) {
	i.remove();
      } // end of if ()
    } // end of while ()
    r.setCompareAll(b);
  }

  public int size() {
    return pairSet.size();
  }

  public Iterator iterator() {
    return pairSet.iterator();
  }

  public String toString() {
    StringBuffer s = new StringBuffer();
    Iterator i = pairSet.iterator();
    while ( i.hasNext() ) {
      s.append(((AttributeValuePairs)i.next()).ruleString());
      s.append("\n");
    } // end of while ()
    return s.toString();
  }

}





