/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *    Reducts.java
 *    Copyright (C) 2002 Ola Leifler
 *
 */

package weka.roughset;

import java.io.*;
import java.util.*;
import weka.core.*;

public class Reducts extends Object implements Cloneable {

  private HashSet reducts;
  private boolean compareDataSets = true;
  private Instances lastSubtable;
  private int numSubtables = 0;

  public Reducts(){
    reducts = new HashSet();
    lastSubtable = null;
    numSubtables = 0;
  }

  public void add(Reduct r) {
    r.setReductSet(this);
    reducts.add(r);
    if ( lastSubtable != r.getInstances() ) {
      lastSubtable = r.getInstances();
      numSubtables++;
    } // end of if ()
  }

  public void filter(double stability){
    
   for (Iterator iter = this.iterator();iter.hasNext(); ) {
     if ( ((Reduct)iter.next()).stability() < stability ) {
       iter.remove();
     } // end of if ()
   } // end of for ()
  }


  public boolean containsSubsetOf(Reduct r) {
    boolean subset = false;
    for (Iterator iter=reducts.iterator();iter.hasNext()&&!subset;) {
      subset = ((Reduct)iter.next()).subsetOf(r);
    } // end of for ()
    return subset;
  }

  /**
   * Get the value of reducts.
   * @return value of reducts.
   */
  public HashSet getReducts() {
    return reducts;
  }
  
  /**
   * Set the value of reducts.
   * @param v  Value to assign to reducts.
   */
  public void setReducts(HashSet  v) {
    this.reducts = v;
  }
  
  
  /**
   * Get the value of compareDataSets.
   * @return value of compareDataSets.
   */
  public boolean getCompareDataSets() {
    return compareDataSets;
  }
  
  /**
   * Set the value of compareDataSets.
   * @param v  Value to assign to compareDataSets.
   */
  public void setCompareDataSets(boolean  v) {
    this.compareDataSets = v;
  }

  /**
   * Get the value of numSubtables.
   * @return value of numSubtables.
   */
  public int getNumSubtables() {
    return numSubtables;
  }
  
  /**
   * Set the value of numSubtables.
   * @param v  Value to assign to numSubtables.
   */
  public void setNumSubtables(int  v) {
    this.numSubtables = v;
  }
  
  /**
   * Get the value of lastSubtable.
   * @return value of lastSubtable.
   */
  public Instances getLastSubtable() {
    return lastSubtable;
  }
  
  /**
   * Set the value of lastSubtable.
   * @param v  Value to assign to lastSubtable.
   */
  public void setLastSubtable(Instances  v) {
    this.lastSubtable = v;
  }
  
  public Iterator iterator() {
    return reducts.iterator();
  }

  public Reduct get(int i){
    int j=0;
    Reduct res = null;
    for (Iterator iter=iterator();iter.hasNext()&&j<=i;j++) {
      res = (Reduct)iter.next();
    } // end of for ()
    return res;
  }
  
  public int size(){
    return reducts.size();
  }

  /* How many reducts in this reduct set are also reducts of the set i ? */

  public int numProperReducts(Instances i){
    int sum=0;
    for (Iterator iter=iterator();iter.hasNext();) {
      sum += (Reduct.isProperReduct(((Reduct)iter.next()).getAttributes(),i)?1:0);
    } // end of for ()
    return sum;
  }

  public Object clone() {
    Reducts copy = new Reducts();
    copy.setReducts((HashSet)reducts.clone());
    copy.setLastSubtable(lastSubtable);
    copy.setNumSubtables(numSubtables);
    return copy;
  }

  public String toString() {
    StringBuffer s = new StringBuffer();
    Iterator i = iterator();
    while ( i.hasNext() ) {
      s.append(((Reduct)i.next()).toString() + "\n");
    } // end of while ()
    return s.toString();
  }

  public String toInfoString(){
    StringBuffer s = new StringBuffer();
    Iterator i = iterator();
    while ( i.hasNext() ) {
      Reduct r = (Reduct)i.next(); 
      s.append(r.toInfoString() + "\n");
    } // end of while ()
    return s.toString();    
  }

}
